@@ -29,9 +29,6 @@ class ElemwiseSuite:
     def time_mul(self):
         self.x * self.y
 
-    def time_index(self):
-        self.x[5]
-
 
 class ElemwiseBroadcastingSuite:
     def setup(self):
@@ -49,11 +46,15 @@ class ElemwiseBroadcastingSuite:
 class IndexingSuite:
     def setup(self):
         np.random.seed(0)
+        self.index = np.random.randint(0, 100, 50)
         self.x = sparse.random((100, 100, 100), density=0.01)
-        self.x[5]  # Numba compilation
+
+        # Numba compilation
+        self.x[5]
+        self.x[self.index]
 
     def time_index_scalar(self):
-        self.x[5]
+        self.x[5, 5, 5]
 
     def time_index_slice(self):
         self.x[:50]
@@ -63,3 +64,6 @@ class IndexingSuite:
 
     def time_index_slice3(self):
         self.x[:50, :50, :50]
+
+    def time_index_fancy(self):
+        self.x[self.index]
