@@ -12,9 +12,15 @@ from numpy.lib.mixins import NDArrayOperatorsMixin
 
 from .._common import dot, matmul
 from .indexing import getitem
-from .umath import elemwise, broadcast_to
+from .._umath import elemwise, broadcast_to
 from .._sparse_array import SparseArray, _reduce_super_ufunc
-from .._utils import normalize_axis, equivalent, check_zero_fill_value, _zero_of_dtype
+from .._utils import (
+    normalize_axis,
+    equivalent,
+    check_zero_fill_value,
+    _zero_of_dtype,
+    can_store,
+)
 
 
 class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
@@ -205,14 +211,19 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         prune=False,
         cache=False,
         fill_value=None,
+        idx_dtype=None,
     ):
         self._cache = None
         if cache:
             self.enable_caching()
 
         if data is None:
-            arr = as_coo(coords, shape=shape, fill_value=fill_value)
+            arr = as_coo(
+                coords, shape=shape, fill_value=fill_value, idx_dtype=idx_dtype
+            )
             self._make_shallow_copy_of(arr)
+            if cache:
+                self.enable_caching()
             return
 
         self.data = np.asarray(data)
@@ -232,7 +243,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
         if shape and not self.coords.size:
             self.coords = np.zeros(
-                (len(shape) if isinstance(shape, Iterable) else 1, 0), dtype=np.uint64
+                (len(shape) if isinstance(shape, Iterable) else 1, 0), dtype=np.intp
             )
 
         if shape is None:
@@ -241,8 +252,18 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             else:
                 shape = ()
 
+        if not isinstance(shape, Iterable):
+            shape = (shape,)
+
         super().__init__(shape, fill_value=fill_value)
-        self.coords = self.coords.astype(np.intp, copy=False)
+        if idx_dtype:
+            if not can_store(idx_dtype, max(shape)):
+                raise ValueError(
+                    "cannot cast array with shape {} to dtype {}.".format(
+                        shape, idx_dtype
+                    )
+                )
+            self.coords = self.coords.astype(idx_dtype)
 
         if self.shape:
             if len(self.data) != self.coords.shape[1]:
@@ -306,11 +327,6 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         """
         return _copy.deepcopy(self) if deep else _copy.copy(self)
 
-    def _make_shallow_copy_of(self, other):
-        self.coords = other.coords
-        self.data = other.data
-        super().__init__(other.shape, fill_value=other.fill_value)
-
     def enable_caching(self):
         """Enable caching of reshape, transpose, and tocsr/csc operations
 
@@ -335,7 +351,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         self._cache = defaultdict(lambda: deque(maxlen=3))
 
     @classmethod
-    def from_numpy(cls, x, fill_value=None):
+    def from_numpy(cls, x, fill_value=None, idx_dtype=None):
         """
         Convert the given :obj:`numpy.ndarray` to a :obj:`COO` object.
 
@@ -382,6 +398,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             has_duplicates=False,
             sorted=True,
             fill_value=fill_value,
+            idx_dtype=idx_dtype,
         )
 
     def todense(self):
@@ -621,7 +638,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         >>> coords = np.random.randint(1000, size=(3, 6), dtype=np.uint16)
         >>> s = COO(coords, data, shape=(1000, 1000, 1000))
         >>> s.nbytes
-        150
+        42
         """
         return self.data.nbytes + self.coords.nbytes
 
@@ -692,255 +709,6 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
 
         return out.reshape(tuple(self.shape[d] for d in neg_axis))
 
-    def mean(self, axis=None, keepdims=False, dtype=None, out=None):
-        """
-        Compute the mean along the given axes. Uses all axes by default.
-
-        Parameters
-        ----------
-        axis : Union[int, Iterable[int]], optional
-            The axes along which to compute the mean. Uses all axes by default.
-        keepdims : bool, optional
-            Whether or not to keep the dimensions of the original array.
-        dtype: numpy.dtype
-            The data type of the output array.
-
-        Returns
-        -------
-        COO
-            The reduced output sparse array.
-
-        See Also
-        --------
-        numpy.ndarray.mean : Equivalent numpy method.
-        scipy.sparse.coo_matrix.mean : Equivalent Scipy method.
-
-        Notes
-        -----
-        * This function internally calls :obj:`COO.sum_duplicates` to bring the
-          array into canonical form.
-        * The :code:`out` parameter is provided just for compatibility with
-          Numpy and isn't actually supported.
-
-        Examples
-        --------
-        You can use :obj:`COO.mean` to compute the mean of an array across any
-        dimension.
-
-        >>> x = np.array([[1, 2, 0, 0],
-        ...               [0, 1, 0, 0]], dtype='i8')
-        >>> s = COO.from_numpy(x)
-        >>> s2 = s.mean(axis=1)
-        >>> s2.todense()  # doctest: +SKIP
-        array([0.5, 1.5, 0., 0.])
-
-        You can also use the :code:`keepdims` argument to keep the dimensions
-        after the mean.
-
-        >>> s3 = s.mean(axis=0, keepdims=True)
-        >>> s3.shape
-        (1, 4)
-
-        You can pass in an output datatype, if needed.
-
-        >>> s4 = s.mean(axis=0, dtype=np.float16)
-        >>> s4.dtype
-        dtype('float16')
-
-        By default, this reduces the array down to one number, computing the
-        mean along all axes.
-
-        >>> s.mean()
-        0.5
-        """
-        if axis is None:
-            axis = tuple(range(self.ndim))
-        elif not isinstance(axis, tuple):
-            axis = (axis,)
-        den = reduce(operator.mul, (self.shape[i] for i in axis), 1)
-
-        if dtype is None:
-            if issubclass(self.dtype.type, (np.integer, np.bool_)):
-                dtype = inter_dtype = np.dtype("f8")
-            else:
-                dtype = self.dtype
-                inter_dtype = (
-                    np.dtype("f4") if issubclass(dtype.type, np.float16) else dtype
-                )
-        else:
-            inter_dtype = dtype
-
-        num = self.sum(axis=axis, keepdims=keepdims, dtype=inter_dtype)
-
-        if num.ndim:
-            out = np.true_divide(num, den, casting="unsafe")
-            return out.astype(dtype) if out.dtype != dtype else out
-        return np.divide(num, den, dtype=dtype, out=out)
-
-    def var(self, axis=None, dtype=None, out=None, ddof=0, keepdims=False):
-        """
-        Compute the variance along the gi66ven axes. Uses all axes by default.
-
-        Parameters
-        ----------
-        axis : Union[int, Iterable[int]], optional
-            The axes along which to compute the variance. Uses all axes by default.
-        dtype : numpy.dtype, optional
-            The output datatype.
-        out: COO, optional
-            The array to write the output to.
-        ddof: int
-            The degrees of freedom.
-        keepdims : bool, optional
-            Whether or not to keep the dimensions of the original array.
-
-        Returns
-        -------
-        COO
-            The reduced output sparse array.
-
-        See Also
-        --------
-        numpy.ndarray.var : Equivalent numpy method.
-
-        Notes
-        -----
-        * This function internally calls :obj:`COO.sum_duplicates` to bring the
-          array into canonical form.
-
-        Examples
-        --------
-        You can use :obj:`COO.var` to compute the variance of an array across any
-        dimension.
-
-        >>> x = np.array([[1, 2, 0, 0],
-        ...               [0, 1, 0, 0]], dtype='i8')
-        >>> s = COO.from_numpy(x)
-        >>> s2 = s.var(axis=1)
-        >>> s2.todense()  # doctest: +SKIP
-        array([0.6875, 0.1875])
-
-        You can also use the :code:`keepdims` argument to keep the dimensions
-        after the variance.
-
-        >>> s3 = s.var(axis=0, keepdims=True)
-        >>> s3.shape
-        (1, 4)
-
-        You can pass in an output datatype, if needed.
-
-        >>> s4 = s.var(axis=0, dtype=np.float16)
-        >>> s4.dtype
-        dtype('float16')
-
-        By default, this reduces the array down to one number, computing the
-        variance along all axes.
-
-        >>> s.var()
-        0.5
-        """
-        axis = normalize_axis(axis, self.ndim)
-
-        if axis is None:
-            axis = tuple(range(self.ndim))
-
-        if not isinstance(axis, tuple):
-            axis = (axis,)
-
-        rcount = reduce(operator.mul, (self.shape[a] for a in axis), 1)
-        # Make this warning show up on top.
-        if ddof >= rcount:
-            warnings.warn("Degrees of freedom <= 0 for slice", RuntimeWarning)
-
-        # Cast bool, unsigned int, and int to float64 by default
-        if dtype is None and issubclass(self.dtype.type, (np.integer, np.bool_)):
-            dtype = np.dtype("f8")
-
-        arrmean = self.sum(axis, dtype=dtype, keepdims=True)
-        np.divide(arrmean, rcount, out=arrmean)
-        x = self - arrmean
-        if issubclass(self.dtype.type, np.complexfloating):
-            x = x.real * x.real + x.imag * x.imag
-        else:
-            x = np.multiply(x, x, out=x)
-
-        ret = x.sum(axis=axis, dtype=dtype, out=out, keepdims=keepdims)
-
-        # Compute degrees of freedom and make sure it is not negative.
-        rcount = max([rcount - ddof, 0])
-
-        ret = ret[...]
-        np.divide(ret, rcount, out=ret, casting="unsafe")
-        return ret[()]
-
-    def std(self, axis=None, dtype=None, out=None, ddof=0, keepdims=False):
-        """
-        Compute the standard deviation along the given axes. Uses all axes by default.
-
-        Parameters
-        ----------
-        axis : Union[int, Iterable[int]], optional
-            The axes along which to compute the standard deviation. Uses
-            all axes by default.
-        dtype : numpy.dtype, optional
-            The output datatype.
-        out: COO, optional
-            The array to write the output to.
-        ddof: int
-            The degrees of freedom.
-        keepdims : bool, optional
-            Whether or not to keep the dimensions of the original array.
-
-        Returns
-        -------
-        COO
-            The reduced output sparse array.
-
-        See Also
-        --------
-        numpy.ndarray.std : Equivalent numpy method.
-
-        Notes
-        -----
-        * This function internally calls :obj:`COO.sum_duplicates` to bring the
-          array into canonical form.
-
-        Examples
-        --------
-        You can use :obj:`COO.std` to compute the standard deviation of an array
-        across any dimension.
-
-        >>> x = np.array([[1, 2, 0, 0],
-        ...               [0, 1, 0, 0]], dtype='i8')
-        >>> s = COO.from_numpy(x)
-        >>> s2 = s.std(axis=1)
-        >>> s2.todense()  # doctest: +SKIP
-        array([0.8291562, 0.4330127])
-
-        You can also use the :code:`keepdims` argument to keep the dimensions
-        after the standard deviation.
-
-        >>> s3 = s.std(axis=0, keepdims=True)
-        >>> s3.shape
-        (1, 4)
-
-        You can pass in an output datatype, if needed.
-
-        >>> s4 = s.std(axis=0, dtype=np.float16)
-        >>> s4.dtype
-        dtype('float16')
-
-        By default, this reduces the array down to one number, computing the
-        standard deviation along all axes.
-
-        >>> s.std()  # doctest: +SKIP
-        0.7071067811865476
-        """
-        ret = self.var(axis=axis, dtype=dtype, out=out, ddof=ddof, keepdims=keepdims)
-
-        ret = np.sqrt(ret)
-        return ret
-
     def transpose(self, axes=None):
         """
         Returns a new array which has the order of the axes switched.
@@ -1099,85 +867,6 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         axes[axis1], axes[axis2] = axes[axis2], axes[axis1]
         return self.transpose(axes)
 
-    @property
-    def real(self):
-        """The real part of the array.
-
-        Examples
-        --------
-        >>> x = COO.from_numpy([1 + 0j, 0 + 1j])
-        >>> x.real.todense()  # doctest: +SKIP
-        array([1., 0.])
-        >>> x.real.dtype
-        dtype('float64')
-
-        Returns
-        -------
-        out : COO
-            The real component of the array elements. If the array dtype is
-            real, the dtype of the array is used for the output. If the array
-            is complex, the output dtype is float.
-
-        See Also
-        --------
-        numpy.ndarray.real : NumPy equivalent attribute.
-        numpy.real : NumPy equivalent function.
-        """
-        return self.__array_ufunc__(np.real, "__call__", self)
-
-    @property
-    def imag(self):
-        """The imaginary part of the array.
-
-        Examples
-        --------
-        >>> x = COO.from_numpy([1 + 0j, 0 + 1j])
-        >>> x.imag.todense()  # doctest: +SKIP
-        array([0., 1.])
-        >>> x.imag.dtype
-        dtype('float64')
-
-        Returns
-        -------
-        out : COO
-            The imaginary component of the array elements. If the array dtype
-            is real, the dtype of the array is used for the output. If the
-            array is complex, the output dtype is float.
-
-        See Also
-        --------
-        numpy.ndarray.imag : NumPy equivalent attribute.
-        numpy.imag : NumPy equivalent function.
-        """
-        return self.__array_ufunc__(np.imag, "__call__", self)
-
-    def conj(self):
-        """Return the complex conjugate, element-wise.
-
-        The complex conjugate of a complex number is obtained by changing the
-        sign of its imaginary part.
-
-        Examples
-        --------
-        >>> x = COO.from_numpy([1 + 2j, 2 - 1j])
-        >>> res = x.conj()
-        >>> res.todense()  # doctest: +SKIP
-        array([1.-2.j, 2.+1.j])
-        >>> res.dtype
-        dtype('complex128')
-
-        Returns
-        -------
-        out : COO
-            The complex conjugate, with same dtype as the input.
-
-        See Also
-        --------
-        numpy.ndarray.conj : NumPy equivalent method.
-        numpy.conj : NumPy equivalent function.
-        """
-        return np.conj(self)
-
     def dot(self, other):
         """
         Performs the equivalent of :code:`x.dot(y)` for :obj:`COO`.
@@ -1328,9 +1017,6 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             extra = int(self.size / np.prod([d for d in shape if d != -1]))
             shape = tuple([d if d != -1 else extra for d in shape])
 
-        if self.shape == shape:
-            return self
-
         if self.size != reduce(operator.mul, shape, 1):
             raise ValueError(
                 "cannot reshape array of size {} into shape {}".format(self.size, shape)
@@ -1344,7 +1030,10 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         # TODO: this self.size enforces a 2**64 limit to array size
         linear_loc = self.linear_loc()
 
-        coords = np.empty((len(shape), self.nnz), dtype=np.intp)
+        idx_dtype = self.coords.dtype
+        if shape != () and not can_store(idx_dtype, max(shape)):
+            idx_dtype = np.min_scalar_type(max(shape))
+        coords = np.empty((len(shape), self.nnz), dtype=idx_dtype)
         strides = 1
         for i, d in enumerate(shape[::-1]):
             coords[-(i + 1), :] = (linear_loc // strides) % d
@@ -1364,7 +1053,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
             self._cache["reshape"].append((shape, result))
         return result
 
-    def resize(self, *args, refcheck=True):
+    def resize(self, *args, refcheck=True, coords_dtype=np.intp):
         """
         This method changes the shape and size of an array in-place.
         Parameters
@@ -1377,6 +1066,9 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         numpy.ndarray.resize : The equivalent Numpy function.
 
         """
+        warnings.warn(
+            "resize is deprecated on all SpraseArray objects.", DeprecationWarning
+        )
         if len(args) == 1 and isinstance(args[0], tuple):
             shape = args[0]
         elif all(isinstance(arg, int) for arg in args):
@@ -1394,7 +1086,10 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         end_idx = np.searchsorted(linear_loc, new_size, side="left")
         linear_loc = linear_loc[:end_idx]
 
-        coords = np.empty((len(shape), len(linear_loc)), dtype=np.intp)
+        idx_dtype = self.coords.dtype
+        if shape != () and not can_store(idx_dtype, max(shape)):
+            idx_dtype = np.min_scalar_type(max(shape))
+        coords = np.empty((len(shape), len(linear_loc)), dtype=idx_dtype)
         strides = 1
         for i, d in enumerate(shape[::-1]):
             coords[-(i + 1), :] = (linear_loc // strides) % d
@@ -1549,7 +1244,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         >>> s = COO(coords, data)
         >>> s._sort_indices()
         >>> s.coords  # doctest: +NORMALIZE_WHITESPACE
-        array([[0, 1, 2]])
+        array([[0, 1, 2]], dtype=uint8)
         >>> s.data  # doctest: +NORMALIZE_WHITESPACE
         array([3, 4, 1], dtype=uint8)
         """
@@ -1577,7 +1272,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         >>> s = COO(coords, data)
         >>> s._sum_duplicates()
         >>> s.coords  # doctest: +NORMALIZE_WHITESPACE
-        array([[0, 1, 2]])
+        array([[0, 1, 2]], dtype=uint8)
         >>> s.data  # doctest: +NORMALIZE_WHITESPACE
         array([6, 7, 2], dtype=uint8)
         """
@@ -1642,62 +1337,6 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         """
         return broadcast_to(self, shape)
 
-    def round(self, decimals=0, out=None):
-        """
-        Evenly round to the given number of decimals.
-
-        See also
-        --------
-        :obj:`numpy.round` : NumPy equivalent ufunc.
-        :obj:`COO.elemwise`: Apply an arbitrary element-wise function to one or two
-            arguments.
-        """
-        if out is not None and not isinstance(out, tuple):
-            out = (out,)
-        return self.__array_ufunc__(
-            np.round, "__call__", self, decimals=decimals, out=out
-        )
-
-    round_ = round
-
-    def clip(self, min=None, max=None, out=None):
-        """
-        Clip (limit) the values in the array.
-
-        Return an array whose values are limited to ``[min, max]``. One of min
-        or max must be given.
-
-        See Also
-        --------
-        sparse.clip : For full documentation and more details.
-        numpy.clip : Equivalent NumPy function.
-        """
-        if min is None and max is None:
-            raise ValueError("One of max or min must be given.")
-        if out is not None and not isinstance(out, tuple):
-            out = (out,)
-        return self.__array_ufunc__(
-            np.clip, "__call__", self, a_min=min, a_max=max, out=out
-        )
-
-    def astype(self, dtype, casting="unsafe", copy=True):
-        """
-        Copy of the array, cast to a specified type.
-
-        See also
-        --------
-        scipy.sparse.coo_matrix.astype : SciPy sparse equivalent function
-        numpy.ndarray.astype : NumPy equivalent ufunc.
-        :obj:`COO.elemwise`: Apply an arbitrary element-wise function to one or two
-            arguments.
-        """
-        # this matches numpy's behavior
-        if self.dtype == dtype and not copy:
-            return self
-        return self.__array_ufunc__(
-            np.ndarray.astype, "__call__", self, dtype=dtype, copy=copy, casting=casting
-        )
-
     def maybe_densify(self, max_size=1000, min_density=0.25):
         """
         Converts this :obj:`COO` array to a :obj:`numpy.ndarray` if not too
@@ -1815,7 +1454,7 @@ class COO(SparseArray, NDArrayOperatorsMixin):  # lgtm [py/missing-equals]
         raise NotImplementedError("The given format is not supported.")
 
 
-def as_coo(x, shape=None, fill_value=None):
+def as_coo(x, shape=None, fill_value=None, idx_dtype=None):
     """
     Converts any given format to :obj:`COO`. See the "See Also" section for details.
 
@@ -1854,7 +1493,7 @@ def as_coo(x, shape=None, fill_value=None):
         return x.asformat("coo")
 
     if isinstance(x, np.ndarray):
-        return COO.from_numpy(x, fill_value=fill_value)
+        return COO.from_numpy(x, fill_value=fill_value, idx_dtype=idx_dtype)
 
     if isinstance(x, scipy.sparse.spmatrix):
         return COO.from_scipy_sparse(x)
